-- Assignments.lua
-- author: rafftnix 
-- date: 09.07.2013

-- Keine Vernderung ohne meine Erlaubnis!
-- No modification without my permission!
-- history: 
--  		v1.0 August 2013 - GIANTS mod contest hand in
-- 			v1.1 November 2015 - field work assignments mod

Assignments = {}

local Assignments_mt = Class(Assignments);

function Assignments:new()
	local self = {};
	setmetatable(self, Assignments_mt);
	
	-- register assignments types
	self.cultivating = FieldWorkAssignments.registerAssignment(self.getFieldAreaToCultivate, self.checkVehiclesForCultivating, g_i18n:getText("cultivating"), g_i18n:getText("cultivatingBriefing"), 5000, nil, nil, nil, self.startCultivatingAssignment, nil, self.basicCleanUp);
	self.ploughing = FieldWorkAssignments.registerAssignment(self.getFieldAreaToPlough, self.checkVehiclesForPloughing, g_i18n:getText("ploughing"), g_i18n:getText("ploughingBriefing"), 12000, nil, nil, nil, self.startCultivatingAssignment, nil, self.basicCleanUp);
	self.seeding = FieldWorkAssignments.registerAssignment(self.getFieldAreaToSeed, self.checkVehiclesForSeeding, g_i18n:getText("seeding"), g_i18n:getText("seedingBriefing"), 5000, {"wheat", "barley", "rape", "maize"}, nil, nil, self.startSeedingAssignment, nil, self.basicCleanUp); 
	self.spraying = FieldWorkAssignments.registerAssignment(self.getFieldAreaToSpray, self.checkVehiclesForSpraying, g_i18n:getText("spraying"), g_i18n:getText("sprayingBriefing"), 2500, nil, nil, nil, self.startSprayingAssignment, nil, self.basicCleanUp); 
	self.threshing = FieldWorkAssignments.registerAssignment(self.getFieldAreaToThresh, self.checkVehiclesForThreshing, g_i18n:getText("threshing"), g_i18n:getText("threshingBriefing"), 5500, {"wheat", "barley", "rape", "maize"}, self.checkThreshDelivery, self.getThreshDeliveryStation, self.startThreshingAssignment, nil, self.basicCleanUp);
	self.chopping = FieldWorkAssignments.registerAssignment(self.getFieldAreaToChop, self.checkVehiclesForChopping, g_i18n:getText("chopping"), g_i18n:getText("choppingBriefing"), 6000, {"wheat", "barley", "rape", "maize"}, self.checkChopDelivery, self.getChopDeliveryStation, self.startChoppingAssignment, nil, self.basicCleanUp);
	
	return self;
end;

function Assignments.getFieldAreaOfAllChannels(field)
	local area = 0;
	
	for a=1, table.getn(field.corners) do
		local corner = field.corners[a];
		
		terrainDetailRequiredMask = bitOR(0, 2 ^ g_currentMission.cultivatorChannel);
		terrainDetailRequiredMask = bitOR(terrainDetailRequiredMask, 2 ^ g_currentMission.sowingChannel);
		terrainDetailRequiredMask = bitOR(terrainDetailRequiredMask, 2 ^ g_currentMission.ploughChannel);
		local fruitDesc = FruitUtil.fruitIndexToDesc[1];
		local terrainDetailProhibitedMask = 0;
		local requiredFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
		local requiredMinGrowthState = 0;
		local requiredMaxGrowthState = 0;
		local prohibitedFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
		local prohibitedMinGrowthState = 0;
		local prohibitedMaxGrowthState = 0;
	
		local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], terrainDetailRequiredMask, terrainDetailProhibitedMask, requiredFruitType, requiredMinGrowthState, requiredMaxGrowthState, prohibitedFruitType, prohibitedMinGrowthState, prohibitedMaxGrowthState);
		
		area = area + area1;
	end;
	return area;
end;

function Assignments.getFieldAreaToCultivate(fieldIndex, fruitType)
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
	local area = 0;
	local totalArea = 0;
	
	for a=1, table.getn(field.corners) do
		local corner = field.corners[a];
	
		terrainDetailRequiredMask = bitOR(0, 2 ^ g_currentMission.cultivatorChannel);
		local fruitDesc = FruitUtil.fruitIndexToDesc[1];
		local terrainDetailProhibitedMask = 0;
		local requiredFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
		local requiredMinGrowthState = 0;
		local requiredMaxGrowthState = 0;
		local prohibitedFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
		local prohibitedMinGrowthState = 0;
		local prohibitedMaxGrowthState = 0;
	
		local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], terrainDetailRequiredMask, terrainDetailProhibitedMask, requiredFruitType, requiredMinGrowthState, requiredMaxGrowthState, prohibitedFruitType, prohibitedMinGrowthState, prohibitedMaxGrowthState);
		
		area = area + area1;
		totalArea = totalArea + totalArea1;
	end;
	return area, totalArea, Assignments.getFieldAreaOfAllChannels(field);
end;

function Assignments.getFieldAreaToPlough(fieldIndex, fruitType)
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
	local area = 0;
	local totalArea = 0;
	
	for a=1, table.getn(field.corners) do
		local corner = field.corners[a];
	
		terrainDetailRequiredMask = bitOR(0, 2 ^ g_currentMission.ploughChannel);
		local fruitDesc = FruitUtil.fruitIndexToDesc[1];
		local terrainDetailProhibitedMask = 0;
		local requiredFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
		local requiredMinGrowthState = 0;
		local requiredMaxGrowthState = 0;
		local prohibitedFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
		local prohibitedMinGrowthState = 0;
		local prohibitedMaxGrowthState = 0;
	
		local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], terrainDetailRequiredMask, terrainDetailProhibitedMask, requiredFruitType, requiredMinGrowthState, requiredMaxGrowthState, prohibitedFruitType, prohibitedMinGrowthState, prohibitedMaxGrowthState);
		
		area = area + area1;
		totalArea = totalArea + totalArea1;
	end;
	return area, totalArea, Assignments.getFieldAreaOfAllChannels(field);
end;

function Assignments.checkVehiclesForCultivating(fruitType)
	local hasCultivator = false;
	local hasTractor = false;
	for a=1, table.getn(g_currentMission.vehicles) do
		local vehicle = g_currentMission.vehicles[a];
		if SpecializationUtil.hasSpecialization(Cultivator, vehicle.specializations) then
			hasCultivator = true;
		end;
		if SpecializationUtil.hasSpecialization(AITractor, vehicle.specializations) then
			hasTractor = true;
		end;
		if hasCultivator and hasTractor then
			return true;
		end;
	end;
	return false;
end;

function Assignments.checkVehiclesForPloughing(fruitType)
	local hasTractor = false;
	local hasPlough = false;
	for a=1, table.getn(g_currentMission.vehicles) do
		local vehicle = g_currentMission.vehicles[a];
		if SpecializationUtil.hasSpecialization(Plough, vehicle.specializations) then
			hasPlough = true;
		end;
		if SpecializationUtil.hasSpecialization(AITractor, vehicle.specializations) then
			hasTractor = true;
		end;
		if hasTractor and hasPlough then
			return true;
		end;
	end;
	return false;
end;

function Assignments.startCultivatingAssignment(assignment)
	local seedsFruitType = FruitUtil.fruitTypes["wheat"].index;

	local areas = {}
	for a=1, table.getn(assignment.field.corners) do
		local corner = assignment.field.corners[a];
		local area = {corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]}
	
		table.insert(areas, area);
		Utils.destroyOtherFruit(FruitUtil.fruitTypes["wheat"].index, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
		Utils.destroyOtherFruit(FruitUtil.fruitTypes["barley"].index, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
	end;
	SowingMachineAreaEvent.runLocally(areas, seedsFruitType, 1, true);	
end;

function Assignments.getFieldAreaToSeed(fieldIndex, fruitType)
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
	
	local area = 0;
	local totalArea = 0;
	
	local requiredFruitType = fruitType;
	
	for a=1, table.getn(field.corners) do
		local corner = field.corners[a];

		local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 2 ^ g_currentMission.sowingChannel, nil, requiredFruitType, 0, 0, 0, 0, 0);
		area = area + area1;
		totalArea = totalArea + totalArea1;
	end;
	
	return area, totalArea, Assignments.getFieldAreaOfAllChannels(field);
end;

function Assignments.checkVehiclesForSeeding(fruitType)
	local hasSeeder = false;
	local hasTractor = false;
	for a=1, table.getn(g_currentMission.vehicles) do
		local vehicle = g_currentMission.vehicles[a];
		if SpecializationUtil.hasSpecialization(AITractor, vehicle.specializations) then
			hasTractor = true;
		end;
		if SpecializationUtil.hasSpecialization(SowingMachine, vehicle.specializations) then
			if vehicle.seeds ~= nil then 
				if fruitType ~= nil then
					for a=1, table.getn(vehicle.seeds) do
						if vehicle.seeds[a] == fruitType then
							hasSeeder = true;
							break;
						end;					
					end;
				else
					hasSeeder = true;
				end;
			end;
		end;
		if hasSeeder and hasTractor then
			return true;
		end;
	end;
	return false;
end;

function Assignments.startSeedingAssignment(assignment)
	Assignments.basicCleanUp(assignment);
end;

function Assignments.getFieldAreaToSpray(fieldIndex, fruitType)
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
	
	local area = 0;
	local totalArea = 0;
	
	for a=1, table.getn(field.corners) do
		local corner = field.corners[a];

		local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 2 ^ g_currentMission.sprayChannel, nil, FruitUtil.FRUITTYPE_UNKNOWN, 0, 0, 0, 0, 0);
		area = area + area1;
		totalArea = totalArea + totalArea1;
	end;
	
	return area, totalArea, Assignments.getFieldAreaOfAllChannels(field);
end;

function Assignments.checkVehiclesForSpraying(fruitType)
	for a=1, table.getn(g_currentMission.vehicles) do
		local vehicle = g_currentMission.vehicles[a];
		if SpecializationUtil.hasSpecialization(Sprayer, vehicle.specializations) then
			return true;
		end;
	end;
	return false;
end;

function Assignments.startSprayingAssignment(assignment)
	local possibleFruits = {"wheat", "barley", "rape", "maize"}
	local fruit = possibleFruits[math.random(1, table.getn(possibleFruits))];
	local fruitDesc = FruitUtil.fruitTypes[fruit];
	
	for a=1, table.getn(assignment.field.corners) do
		local corner = assignment.field.corners[a];
		
		local x, z, widthX, widthZ, heightX, heightZ = Utils.getXZWidthAndHeight(g_currentMission.terrainDetailId, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
	
		local state = fruitDesc.minHarvestingGrowthState-1;
		local numChannels = g_currentMission.numFruitStateChannels; 
		Utils.updateSowingArea(fruitDesc.index, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 1, true);
		setDensityParallelogram(g_currentMission.fruits[fruitDesc.index].id, x, z, widthX, widthZ, heightX, heightZ, 0, numChannels, state);
		setDensityParallelogram(g_currentMission.terrainDetailId, x, z, widthX, widthZ, heightX, heightZ, g_currentMission.sprayChannel, 1, 0);
	end;
end;

function Assignments.getFieldAreaToThresh(fieldIndex, fruitType)
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
	
	local area = 0;
	local totalArea = 0;

	local fruitDesc = FruitUtil.fruitIndexToDesc[fruitType];
	if fruitDesc ~= nil then
		for a=1, table.getn(field.corners) do
			local corner = field.corners[a];
			local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 0, 0, fruitType, fruitDesc.maxHarvestingGrowthState+1, fruitDesc.cutState, fruitType, fruitDesc.minHarvestingGrowthState, fruitDesc.maxHarvestingGrowthState);	
			--local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 0, 0, fruitType, fruitDesc.minHarvestingGrowthState, fruitDesc.maxHarvestingGrowthState, fruitType, fruitDesc.maxHarvestingGrowthState+1, fruitDesc.cutState);	
			area = area + area1;
			totalArea = totalArea + totalArea1;
		end;
	end;
	
	return area, totalArea, Assignments.getFieldAreaOfAllChannels(field);
end;

function Assignments.checkVehiclesForThreshing(fruitType)
	local fillType;
	if fruitType ~= nil then
		fillType = FruitUtil.fruitTypeToFillType[fruitType];
	end;
	
	local combineAllowed = false;
	local cutterAllowed = false;
	local trailerAllowed = false;
	local tractorAllowed = false;
	
	for a=1, table.getn(g_currentMission.vehicles) do
		local vehicle = g_currentMission.vehicles[a];
		
		if SpecializationUtil.hasSpecialization(Combine, vehicle.specializations) then
			if fillType ~= nil then
				if vehicle.fillTypes ~= nil and vehicle.fillTypes[fillType] ~= nil then
					if vehicle.fillTypes[fillType] then
						combineAllowed = true;
					end;
				end;
			else
				combineAllowed = true;
			end;
		end;
		
		if SpecializationUtil.hasSpecialization(Cutter, vehicle.specializations) then
			if fruitType ~= nil then
				if vehicle.fruitTypes ~= nil and vehicle.fruitTypes[fruitType] ~= nil then
					if vehicle.fruitTypes[fruitType] then
						cutterAllowed = true;
					end;
				end;
			else
				cutterAllowed = true;
			end;
		end;
		
		if SpecializationUtil.hasSpecialization(Trailer, vehicle.specializations) then
			if fillType ~= nil then
				if vehicle.fillTypes ~= nil and vehicle.fillTypes[fillType] ~= nil then
					if vehicle.fillTypes[fillType] then
						trailerAllowed = true;
					end;
				end;
			else
				trailerAllowed = true;
			end;
		end;
		
		if SpecializationUtil.hasSpecialization(AITractor, vehicle.specializations) then
			tractorAllowed = true;
		end;
	end;
	return combineAllowed and cutterAllowed and trailerAllowed and tractorAllowed;
end;

function Assignments.startThreshingAssignment(assignment)
	for a=1, table.getn(assignment.field.corners) do
		local corner = assignment.field.corners[a];
		
		local x, z, widthX, widthZ, heightX, heightZ = Utils.getXZWidthAndHeight(g_currentMission.fruits[assignment.fruitType].id, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
	
		local desc = FruitUtil.fruitIndexToDesc[assignment.fruitType];
		local state = desc.maxHarvestingGrowthState;
		local numChannels = g_currentMission.numFruitStateChannels; 
		Utils.updateSowingArea(assignment.fruitType, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 0, true);
		if assignment.fruitType ~= FruitUtil.fruitTypes["grass"].index then
			setDensityParallelogram(g_currentMission.terrainDetailId, x, z, widthX, widthZ, heightX, heightZ, g_currentMission.sprayChannel, 1, 1);
		end;
		setDensityParallelogram(g_currentMission.fruits[assignment.fruitType].id, x, z, widthX, widthZ, heightX, heightZ, 0, numChannels, state);
	end;
end;

function Assignments.checkThreshDelivery(assignment) 
	local delivery = assignment.delivery;
	local harvestedAmount = math.max(1, delivery.harvestedAmount); -- avoid dividing with 0
	delivery.progress = (delivery.deliveredAmount/harvestedAmount) * 100;
	local fulFilled = delivery.progress >= 99;
	
	return fulFilled;
end;

function Assignments.getThreshDeliveryStation(fruitType)
	local fillType = FruitUtil.fruitTypeToFillType[fruitType];

	-- find random tip trigger
	local possibleTriggers = {}
	for trigger, sameTrigger in pairs(g_currentMission.tipTriggers) do
		if not trigger.isFarmTrigger and trigger.stationName ~= nil and trigger.stationName ~= "Station" and trigger.acceptedFillTypes[fillType] ~= nil then
			if trigger.doNotUseForAssignments == nil or not trigger.doNotUseForAssignments then
				if trigger.isHTP == nil or not trigger.isHTP then
					table.insert(possibleTriggers, trigger);
				end;
			end;
		end;
	end;
	
	local chosenTrigger = possibleTriggers[math.random(1, table.getn(possibleTriggers))];
	if chosenTrigger ~= nil and chosenTrigger.stationName ~= nil then
		return {chosenTrigger}, chosenTrigger.stationName;
	else
		return {nil}, "";
	end;
end;

function Assignments.getFieldAreaToChop(fieldIndex, fruitType)
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
	
	local area = 0;
	local totalArea = 0;

	local fruitDesc = FruitUtil.fruitIndexToDesc[fruitType];
	if fruitDesc ~= nil then
		for a=1, table.getn(field.corners) do
			local corner = field.corners[a];
			local area1, totalArea1 = AITractor.getAIArea(self, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 0, 0, 												fruitType, 			fruitDesc.maxHarvestingGrowthState, fruitDesc.cutState, 	fruitType, 				fruitDesc.minHarvestingGrowthState, fruitDesc.minHarvestingGrowthState);	
			--AITractor.getAIArea(self, lStartX, lStartZ, lWidthX, lWidthZ, lHeightX, lHeightZ, 								terrainDetailRequiredMask, terrainDetailProhibitedMask, requiredFruitType, requiredMinGrowthState, 				requiredMaxGrowthState, prohibitedFruitType, 	prohibitedMinGrowthState, prohibitedMaxGrowthState)
			area = area + area1;
			totalArea = totalArea + totalArea1;
		end;
	end;
	
	return area, totalArea, Assignments.getFieldAreaOfAllChannels(field);
end;

function Assignments.checkVehiclesForChopping(fruitType)
	local fillType;
	if fruitType ~= nil then
		fillType = Fillable.fillTypeNameToInt["chaff"];
	end;
	
	local combineAllowed = false;
	local cutterAllowed = false;
	local trailerAllowed = false;
	local tractorAllowed = false;
	
	for a=1, table.getn(g_currentMission.vehicles) do
		local vehicle = g_currentMission.vehicles[a];
		
		if SpecializationUtil.hasSpecialization(Combine, vehicle.specializations) then
			if fillType ~= nil then
				if vehicle.fillTypes ~= nil and vehicle.fillTypes[fillType] ~= nil then
					if vehicle.fillTypes[fillType] then
						combineAllowed = true;
					end;
				end;
			else
				combineAllowed = true;
			end;
		end;
		
		if SpecializationUtil.hasSpecialization(Cutter, vehicle.specializations) then
			if fruitType ~= nil then
				if vehicle.fruitTypes ~= nil and vehicle.fruitTypes[fruitType] ~= nil then
					if vehicle.fruitTypes[fruitType] then
						cutterAllowed = true;
					end;
				end;
			else
				cutterAllowed = true;
			end;
		end;
		
		-- trailer
		if SpecializationUtil.hasSpecialization(Trailer, vehicle.specializations) then
			if fillType ~= nil then
				if vehicle.fillTypes ~= nil and vehicle.fillTypes[fillType] ~= nil then
					if vehicle.fillTypes[fillType] then
						trailerAllowed = true;
					end;
				end;
			else
				trailerAllowed = true;
			end;
		end;
		
		if SpecializationUtil.hasSpecialization(AITractor, vehicle.specializations) then
			tractorAllowed = true;
		end;
	end;
	return combineAllowed and cutterAllowed and trailerAllowed and tractorAllowed;
end;

function Assignments.checkChopDelivery(assignment)
	return Assignments.checkThreshDelivery(assignment);
end;

function Assignments.getChopDeliveryStation(fruitType)
	local chaff = Fillable.fillTypeNameToInt["chaff"];
	
	-- find random tip trigger
	local triggers = {}
	for trigger, sameTrigger in pairs(g_currentMission.tipTriggers) do
		if trigger.acceptedFillTypes[chaff] ~= nil and trigger.bunkerSilo ~= nil then 
			if trigger.doNotUseForAssignments == nil or not trigger.doNotUseForAssignments then
				if trigger.isHTP == nil or not trigger.isHTP then
					table.insert(triggers, trigger);
				end;
			end;
		end;
	end;
	
	return triggers, "bioGasPlant";
end;

function Assignments.startChoppingAssignment(assignment)
	for a=1, table.getn(assignment.field.corners) do
		local corner = assignment.field.corners[a];
		
		local x, z, widthX, widthZ, heightX, heightZ = Utils.getXZWidthAndHeight(g_currentMission.fruits[assignment.fruitType].id, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
	
		local desc = FruitUtil.fruitIndexToDesc[assignment.fruitType];
		local state = desc.maxHarvestingGrowthState-1;
		local numChannels = g_currentMission.numFruitStateChannels; 
		Utils.updateSowingArea(assignment.fruitType, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], 0, true);
		if assignment.fruitType ~= FruitUtil.fruitTypes["grass"].index then
			setDensityParallelogram(g_currentMission.terrainDetailId, x, z, widthX, widthZ, heightX, heightZ, g_currentMission.sprayChannel, 1, 1);
		end;
		setDensityParallelogram(g_currentMission.fruits[assignment.fruitType].id, x, z, widthX, widthZ, heightX, heightZ, 0, numChannels, state);
	end;
end;

function Assignments.basicCleanUp(assignment)
	for a=1, table.getn(assignment.field.corners) do
		local corner = assignment.field.corners[a];
		Utils.updateCultivatorArea(corner[3], corner[4], corner[1], corner[2], corner[5], corner[6], false, false, 0);

		-- now remove spray if there is any left
		local x,z, wx, wz, hx, hz = Utils.getXZWidthAndHeight(g_currentMission.terrainDetailId, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
		local _, numPixels = setDensityMaskedParallelogram(g_currentMission.terrainDetailId, x, z, wx, wz, hx, hz, g_currentMission.sprayChannel, 1, g_currentMission.terrainDetailId, g_currentMission.terrainDetailTypeFirstChannel, g_currentMission.terrainDetailTypeNumChannels, 0);
	end;
	
	--[[for a=1, table.getn(assignment.field.corners) do
		local corner = assignment.field.corners[a];
		
		Utils.destroyOtherFruit(FruitUtil.fruitTypes["wheat"].index, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
		Utils.destroyOtherFruit(FruitUtil.fruitTypes["barley"].index, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
		
		local x, z, widthX, widthZ, heightX, heightZ = Utils.getXZWidthAndHeight(g_currentMission.terrainDetailId, corner[3], corner[4], corner[1], corner[2], corner[5], corner[6]);
		setDensityParallelogram(g_currentMission.terrainDetailId, x, z, widthX, widthZ, heightX, heightZ, 0, g_currentMission.numFruitStateChannels, 0);
		setDensityParallelogram(g_currentMission.terrainDetailId, x, z, widthX, widthZ, heightX, heightZ, g_currentMission.sprayChannel, 1, 0);
		setDensityParallelogram(g_currentMission.terrainDetailId, x, z, widthX, widthZ, heightX, heightZ, g_currentMission.terrainDetailTypeFirstChannel, g_currentMission.terrainDetailTypeNumChannels, 2^g_currentMission.cultivatorChannel);
	end;]]
end;