SuggestAssignmentEvent = {};
SuggestAssignmentEvent_mt = Class(SuggestAssignmentEvent, Event);
InitEventClass(SuggestAssignmentEvent, "SuggestAssignmentEvent");

-- dual use event:
	-- if the assignment is successfully generated, it sends the assignment to the admins
	-- if no assignment could be generated, it sends the assignment to the admins

function SuggestAssignmentEvent:emptyNew()
    local self = Event:new(SuggestAssignmentEvent_mt);
    self.className = "SuggestAssignmentEvent";
    return self;
end;

function SuggestAssignmentEvent:new(fieldIndex, typeIndex, fieldArea, fruitType, stationName)
	local self = SuggestAssignmentEvent:emptyNew();
	self.fieldIndex = fieldIndex;
	self.typeIndex = typeIndex;
	self.fieldArea = fieldArea;
	self.fruitType = fruitType;
	self.stationName = stationName;
    return self;
end;

function SuggestAssignmentEvent:writeStream(streamId, connection)
	if self.fieldIndex == false then -- quick and dirty - use old variables if there is no assignment
		streamWriteBool(streamId, false);
		streamWriteString(streamId, self.typeIndex);
	else
		streamWriteBool(streamId, true);
		streamWriteInt8(streamId, self.fieldIndex);
		streamWriteInt8(streamId, self.typeIndex);
		streamWriteFloat32(streamId, self.fieldArea);
		streamWriteInt8(streamId, self.fruitType);
		streamWriteString(streamId, self.stationName);
	end;
end;

function SuggestAssignmentEvent:readStream(streamId, connection)
	if streamReadBool(streamId) then
		self.fieldIndex = streamReadInt8(streamId);
		self.typeIndex = streamReadInt8(streamId);
		self.fieldArea = streamReadFloat32(streamId);
		self.fruitType = streamReadInt8(streamId);
		self.stationName = streamReadString(streamId);
		self:run(connection);
	else
		g_currentMission.assignmentSettingsHUD:setErrorMessage(streamReadString(streamId));
	end;
end;

function SuggestAssignmentEvent:run(connection)
	local assignment = {}
	assignment.fieldIndex = self.fieldIndex;
	assignment.field = g_currentMission.fieldDefinitionBase.fieldDefs[assignment.fieldIndex];
	assignment.type = AssignmentTypes[self.typeIndex];
	assignment.fruitType = self.fruitType;
	assignment.haToWorkOn = self.fieldArea;
	assignment.price = math.floor(assignment.haToWorkOn * assignment.type.pricePerHa);
	if assignment.type.checkDeliveryFunction ~= nil then
		assignment.delivery = {}
		assignment.delivery.stationName = self.stationName;
		assignment.delivery.surchargeFactor = 0.3;
		assignment.delivery.surcharge = assignment.price * assignment.delivery.surchargeFactor;
	end;
	assignment.briefing = g_currentMission.fieldWorkMissions:getAssignmentBriefing(assignment);
	g_currentMission.assignmentGUI:openAssignment(assignment);
end;