-- assignmentGUI.lua
-- author: rafftnix 
-- date: 21.05.2013

-- Keine Vernderung ohne meine Erlaubnis!
-- No modification without my permission!
-- history: 
--  		v1.0 August 2013 - GIANTS mod contest hand in
--			v1.1 November 2014 - field work missions mod

AssignmentGUI = {}

local AssignmentGUI_mt = Class(AssignmentGUI);

function AssignmentGUI:new()
	local self = {}
	setmetatable(self, AssignmentGUI_mt);

	g_currentMission.assignmentGUI = self;
	
	self.farmerNames = Utils.splitString(" ", g_i18n:getText("farmerNames"));
		
	self.gui = RafftnixGUI:new(nil, true, true);
	
	self.assignmentGUI_Container = self.gui:createElement(self.gui.baseElement, 0, 0, 0, 0);
	self.assignmentGUI_BG = self.gui:createImage(self.assignmentGUI_Container, 0.225, 0.25, 0.6, 0.5, "GUI/choose_screen.dds");
	self.assignmentOfferedText = self.gui:createText(self.assignmentGUI_Container, 0.3, 0.68, 0.3, 0.1, "", 0.03, {1,1,1,1});
	
	self.briefingText = self.gui:createText(self.assignmentGUI_Container, 0.3, 0.55, 0.3, 0.1, "", 0.03, {1,1,1,1});
	self.areaText = self.gui:createText(self.assignmentGUI_Container, 0.3, 0.44, 0.3, 0.1, "", 0.03, {1,1,1,1});
	self.paymentText = self.gui:createText(self.assignmentGUI_Container, 0.3, 0.4, 0.3, 0.1, "", 0.03, {1,1,1,1});

	self.acceptButton = self.gui:createButton(self.assignmentGUI_Container, 0.3, 0.26, 0.22, 0.075, g_i18n:getText("acceptAssignment"), 0.03, {0,0,0,1}, self.acceptAssignment, self);
	self.refuseButton = self.gui:createButton(self.assignmentGUI_Container, 0.53, 0.26, 0.22, 0.075, g_i18n:getText("refuseAssignment"), 0.03, {0,0,0,1}, self.refuseAssignment, self);
	self.acceptButton:setTextFocus("middle", "center");
	self.refuseButton:setTextFocus("middle", "center");	
	return self;
end;

function AssignmentGUI:open()
	self.gui:open();
end;

function AssignmentGUI:openAssignment(assignment)
	if g_currentMission.fieldWorkMissions.assignmentSettingsHUD.gui.baseElement.visibility then
		g_currentMission.fieldWorkMissions.assignmentSettingsHUD:close();
	end;
		
	if not self.gui.baseElement.visibility then
		self:open();
	end;

	self.assignmentGUI_Container:setVisibility(true);
		
	local text = string.gsub(g_i18n:getText("assignmentOffered"), "name", self.farmerNames[math.random(1, table.getn(self.farmerNames))]);
	self.assignmentOfferedText:setText(text);
		
	self.briefingText:setText(assignment.briefing);
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[assignment.fieldIndex];
	local areaText = string.gsub(g_i18n:getText("areaText"), "area1", tostring(math.floor(assignment.haToWorkOn*100)/100)); 
	self.areaText:setText(areaText);
	local paymentText = string.gsub(g_i18n:getText("paymentText"), "price", math.floor(assignment.price), 1); 
	paymentText = string.gsub(paymentText, "pricePerHa", math.floor(assignment.type.pricePerHa)); 	
	if assignment.delivery ~= nil then
		paymentText = paymentText.." "..string.gsub(g_i18n:getText("additionalDeliveryPrice"), "price", math.floor(assignment.delivery.surcharge)); 	
	end;	
	self.paymentText:setText(paymentText);
	
	self.assignmentGUI_Container:setVisibility(true);
end;

function AssignmentGUI:close()
	self.gui:close();
end;

function AssignmentGUI:acceptAssignment()
	if g_server ~= nil then
		g_currentMission.fieldWorkMissions:acceptAssignment(g_currentMission.fieldWorkMissions.suggestedAssignment);
	else -- client
		g_client:getServerConnection():sendEvent(AcceptAssignmentEvent:new());
		self.assignment = nil;
	end;
	self:close();
end;

function AssignmentGUI:refuseAssignment(noEventSend)
	self.assignment = nil;
	if g_server ~= nil then
		g_currentMission.fieldWorkMissions.suggestedAssignment = nil;
	end;
	if not noEventSend then
		g_currentMission.assignmentSettingsHUD:open();
		RefuseAssignmentEvent.sendEvent(noEventSend);
	end;
end;

AssignmentCompletedGUI = {}
local AssignmentCompletedGUI_mt = Class(AssignmentCompletedGUI);

-- GUI which is shown when you have sucessfully finished an assignment

function AssignmentCompletedGUI:new()
	local self = {}
	setmetatable(self, AssignmentCompletedGUI_mt);
	
	g_currentMission.assignmentCompletedGUI = self;

	self.gui = RafftnixGUI:new(nil, true, true);
	
	self.backGroundImage = self.gui:createImage(self.gui.baseElement, 0.2, 0.25, 0.6, 0.5, "GUI/assignmentCompletedGUI.dds");
	self.assignmentFinished = self.gui:createText(self.gui.baseElement, 0.5, 0.68, 0, 0, g_i18n:getText("assignmentFinished"), 0.05, {1,1,1,1});
	self.assignmentFinished:setTextFocus("middle", "bottom");
	self.paymentText = self.gui:createText(self.gui.baseElement, 0.5, 0.5, 0, 0, "", 0.03,  {1,1,1,1});
	self.paymentText:setTextFocus("middle", "bottom");
	self.areaText = self.gui:createText(self.gui.baseElement, 0.5, 0.45, 0, 0, "", 0.03,  {1,1,1,1});
	self.areaText:setTextFocus("middle", "bottom");
	self.quitButton = self.gui:createButton(self.gui.baseElement, 0.26, 0.27, 0.48, 0.08, g_i18n:getText("back"), 0.05,  {0,0,0,1}, self.close, self);
	self.quitButton:setTextFocus("middle", "center");
	return self;
end;

function AssignmentCompletedGUI:open(assignment)
	self.gui:open();
	local paymentText = string.gsub(g_i18n:getText("paymentText"), "price", math.floor(assignment.price * assignment.progress / 100), 1); 
	paymentText = string.gsub (paymentText, "pricePerHa", math.floor(assignment.type.pricePerHa)); 
	if assignment.delivery ~= nil then
		paymentText = paymentText.." "..string.gsub(g_i18n:getText("additionalDeliveryPrice"), "price", math.floor(assignment.delivery.surcharge*assignment.delivery.progress/100)); 	
	end;	
	self.paymentText:setText(paymentText);
	local areaText = string.gsub(g_i18n:getText("areaText"), "area1", math.floor(assignment.haToWorkOn * assignment.progress )/100); 
	self.areaText:setText(areaText);
end;

function AssignmentCompletedGUI:close()
	self.gui:close();
	if (g_server ~= nil and g_client ~= nil) or g_currentMission.users[g_currentMission.playerUserId].isMasterUser then
		g_currentMission.assignmentSettingsHUD:open();
	end;
end;