-- author: rafftnix
-- date: 14.11.2014


AssignmentSettingsHUD = {}

local AssignmentSettingsHUD_mt = Class(AssignmentSettingsHUD);

function AssignmentSettingsHUD:new()
	local self = {};
	setmetatable(self, AssignmentSettingsHUD_mt);
	
	self.gui = RafftnixGUI:new(nil, true, true);
	g_currentMission.assignmentSettingsHUD = self;
	self.gui:createImage(self.gui.baseElement, 0.2, 0.2, 0.6, 0.6, "GUI/settings.dds");
	self.headLine = self.gui:createText(self.gui.baseElement, 0.5, 0.725, 0, 0, g_i18n:getText("fieldWorkMissions"), 0.035, {1,1,1,1});
	self.headLine:setTextFocus("middle", "bottom");
	self.requestAssignmentButton = self.gui:createButton(self.gui.baseElement, 0.2, 0.6, 0.6, 0.05, g_i18n:getText("requestAssignment"), 0.03, {1,1,1,1}, self.requestAssignment, self);
	self.requestAssignmentButton:setTextFocus("middle", "center");
	self.cancelAssignmentButton = self.gui:createButton(self.gui.baseElement, 0.2, 0.55, 0.6, 0.05, g_i18n:getText("cancelAssignment"), 0.03, {1,1,1,1}, self.cancelAssignment, self);
	self.cancelAssignmentButton:setTextFocus("middle", "center");
	self.errorMessage = self.gui:createText(self.gui.baseElement, 0.2, 0.50, 0.6, 0.05, "", 0.03, {1,0,0,1});
	self.errorMessage:setTextFocus("middle", "center");
	self.preferedAssignmentTypeText = self.gui:createText(self.gui.baseElement, 0.45, 0.225, 0.22, 0.075, "Beispiel", 0.03, {1,1,1,1});
	self.preferedAssignmentTypeText:setTextFocus("middle", "center");
	self.arrowLeftButton = self.gui:createButton(self.gui.baseElement, 0.38, 0.225, 0.07, 0.075, "", 0.03, 1, self.assignmentTypeLeft, self);
	self.arrowRightButton = self.gui:createButton(self.gui.baseElement, 0.68, 0.225, 0.07, 0.075, "", 0.03, 1, self.assignmentTypeRight, self);
	
	self.backButton = self.gui:createButton(self.gui.baseElement, 0.25, 0.225, 0.11, 0.075, g_i18n:getText("back"), 0.03, {1, 1, 1, 1}, self.close, self);
	self.backButton:setTextFocus("right", "center");
	
	self.selectionIndex = 0;
	return self;
end;

function AssignmentSettingsHUD:open()
	self.gui:open();
	self:updateHUD();
end;

function AssignmentSettingsHUD:close()
	self.errorMessage:setVisibility(false);
	self.gui:close();
end;

function AssignmentSettingsHUD:requestAssignment()
	self.errorMessage:setVisibility(false);
	
	if g_server ~= nil then
		if g_currentMission.fieldWorkMissions.currentAssignment == nil then
			g_currentMission.fieldWorkMissions:handleAssignmentRequest();
		end;
	else -- beeing master user already confirmed
		g_client:getServerConnection():sendEvent(RequestAssignmentEvent:new());
	end;
end;

function AssignmentSettingsHUD:setErrorMessage(message)
	if self.gui.baseElement.visibility then
		self:open();
	end;
	self.errorMessage:setVisibility(true);
	self.errorMessage:setText(message);
end;

function AssignmentSettingsHUD:cancelAssignment()
	self:updateHUD();
	if g_server ~= nil then
		if g_currentMission.fieldWorkMissions.currentAssignment ~= nil then
			g_currentMission.fieldWorkMissions:cancelAssignment();
		end;
	else
		g_client:getServerConnection():sendEvent(CancelAssignmentEvent:new());
	end;
end;

function AssignmentSettingsHUD:assignmentTypeLeft(noEventSend)
	local index = math.max(0, self.selectionIndex-1);
	self:setSelectedAssignmentTypeIndex(index);
end;

function AssignmentSettingsHUD:assignmentTypeRight()
	local index = math.min(self.selectionIndex+1, table.getn(AssignmentTypes));
	self:setSelectedAssignmentTypeIndex(index);
end;

function AssignmentSettingsHUD:setSelectedAssignmentTypeIndex(index, noEventSend)
	self.selectionIndex = index;
	g_currentMission.fieldWorkMissions:setPreferedAssignmentTypeIndex(index);
	self:updateHUD();
	
	if not noEventSend then
		SetSelectedAssignmentTypeEvent.sendEvent(index, noEventSend);
	end;
end;

function AssignmentSettingsHUD:updateHUD()
	-- isAuthorised is true if you are the server or an admin (dedicated server)
	if g_currentMission.playerUserId ~= nil and g_currentMission.users[g_currentMission.playerUserId] ~= nil then
		local isAuthorised = g_server ~= nil or g_currentMission.users[g_currentMission.playerUserId].isMasterUser;

		self.requestAssignmentButton:setDisabled(g_currentMission.fieldWorkMissions.currentAssignment ~= nil or not isAuthorised);
		self.preferedAssignmentTypeText:setTextColor({1,1,1,1});
		self.cancelAssignmentButton:setDisabled(g_currentMission.fieldWorkMissions.currentAssignment == nil or not isAuthorised);
		self.arrowLeftButton:setDisabled(not isAuthorised);
		self.arrowRightButton:setDisabled(not isAuthorised);
			
		self.errorMessage:setVisibility(false);
			
		if self.selectionIndex == 0 then
			self.preferedAssignmentTypeText:setText(g_i18n:getText("random"));
		else
			self.preferedAssignmentTypeText:setText(AssignmentTypes[self.selectionIndex].name);
		end;
	end;
end;