-- author: rafftnix
-- date: 10.11.2014

FieldWorkAssignments = {}

AssignmentTypes = {}
AssignmentTypes.numRegisteredAssignments = 0;

function FieldWorkAssignments.registerAssignment(areaFunction, checkVehicleFunction, name, description, pricePerHa, possibleFruitTypes, checkDeliveryFunction, getDeliveryStation, startAssignmentCallback, stopAssignmentCallback, cleanUpFunction)
	AssignmentTypes.numRegisteredAssignments = AssignmentTypes.numRegisteredAssignments + 1;
	local assignmentType = {}
	assignmentType.areaFunction = areaFunction;
	assignmentType.checkVehicleFunction = checkVehicleFunction;
	assignmentType.cleanUpFunction = cleanUpFunction;
	assignmentType.name = name;
	assignmentType.description = description;
	if pricePerHa == nil then
		printCallstack();
	end;
	assignmentType.pricePerHa = FieldWorkAssignments.difficultyLevelPayments(pricePerHa, g_currentMission.missionStats.difficulty);
	assignmentType.possibleFruitTypes = possibleFruitTypes;
	assignmentType.index = AssignmentTypes.numRegisteredAssignments;
	
	if checkDeliveryFunction ~= nil then
		assignmentType.checkDeliveryFunction = checkDeliveryFunction;
	end;
	
	if getDeliveryStation ~= nil then
		assignmentType.getDeliveryStation = getDeliveryStation;
	end;
	
	if startAssignmentCallback ~= nil then
		assignmentType.startAssignmentCallback = startAssignmentCallback;
	end;
	
	if stopAssignmentCallback ~= nil then
		assignmentType.stopAssignmentCallback = stopAssignmentCallback;
	end;
		
	table.insert(AssignmentTypes, assignmentType);
	return assignmentType;
end;

function FieldWorkAssignments.difficultyLevelPayments(price, difficultyLevel)
	if difficultyLevel == 1 then
		return price;
	elseif difficultyLevel == 2 then
		return price * 0.5;
	elseif difficultyLevel == 3 then
		return price * 0.2;
	else
		return 0;
	end;
end;

function FieldWorkAssignments:loadMap(name)
	if self.firstRunDone == nil then
		self.firstRunDone = true;
		self.mapSupported = false;
		if name == "data/maps/map01.i3d" or name == "data/maps/map02.i3d" or (g_currentMission.mapSupportsFieldWorkAssignments ~= nil and g_currentMission.mapSupportsFieldWorkAssignments) then
			self.mapSupported = true;
		end;		
		local _, num = string.gsub(name, "pdlc/goldPack/map/map01.i3d", ""); 
		if num > 0 then
			self.mapSupported = true;
		end;		
		
		g_currentMission.fieldWorkMissions = self;
		self.assignments = Assignments:new();
		self.assignmentHUD = AssignmentHUD:new();
		self.assignmentGui = AssignmentGUI:new();
		self.welcomeGUI = Welcome_GUI:new();
		self.assignmentSettingsHUD = AssignmentSettingsHUD:new();
		self.assignmentCompletedGui = AssignmentCompletedGUI:new();
		self.assignmentWorkTolerance = 0.01;
		self.preferedAssignmentTypeIndex = 0; -- 0 means random assignments, > 0 means certain assignmentType 
		self.assignmentsToCleanUp = {}
		self.mpUserbackup = 1;
		self.cleanUpAssignmentsTimer = 0;
	
		local dir = getUserProfileAppPath();
		self.filename = dir.."/assignments.xml"; 
		self.fileFound = false;
	
		getFiles(dir, "getFilesCallback", self);
	
		if self.fileFound then
			local xml = loadXMLFile("assignments", self.filename); 
			self.preferedAssignmentTypeIndex = Utils.getNoNil(getXMLInt(xml, "assignments.preferedAssignmentTypeIndex"), 0);
			if self.preferedAssignmentTypeIndex ~= 0 and not AssignmentTypes[self.preferedAssignmentTypeIndex].checkVehicleFunction() then
				self.preferedAssignmentTypeIndex = 0;
			end;
			delete(xml);
		end;
	
		-- save field corner data
		if g_currentMission ~= nil and g_currentMission.fieldDefinitionBase ~= nil and g_currentMission.fieldDefinitionBase.fieldDefs ~= nil then
			local fields = g_currentMission.fieldDefinitionBase.fieldDefs; 
			for a=1, table.getn(fields) do
				local field = fields[a];
				field.corners = {}
				field.useForAssignments = Utils.getNoNil(getUserAttribute(field.fieldDimensions, "useForAssignments"), true);
				for a=1, getNumOfChildren(field.fieldDimensions) do
					local corner = getChildAt(field.fieldDimensions, a-1);
					local cx, cy, cz = getWorldTranslation(corner);
					local c1x, c1y, c1z = getWorldTranslation(getChildAt(corner, 0));
					local c2x, c2y, c2z = getWorldTranslation(getChildAt(corner, 1));
					table.insert(field.corners, {cx, cz, c1x, c1z, c2x, c2z});
				end; 
			end;
		end;
	end;
end;

function printContent(content, depth)
	if depth < 5 then
		for k, v in pairs(content) do
			local space = "";
			for a=1, depth do
				space = space .. "   ";
			end;

			print(space .. "k: " .. tostring(k));
			
			if type(k) == "table" then
				printContent(k, depth+1)
			end;
			
			print(space .. "v: " .. tostring(v));
			
			if type(v) == "table" then
				printContent(v, depth+1)
			end;
		end;
	end;
end;

function FieldWorkAssignments:getFilesCallback(filename)
	if filename == "assignments.xml" then
		self.fileFound = true;
	end;
end;

function FieldWorkAssignments:deleteMap()
	if not g_currentMission.missionDynamicInfo.isMultiplayer or (g_currentMission.missionDynamicInfo.isMultiplayer and g_server ~= nil) then
		local xml = createXMLFile("assignments", self.filename, "assignments");
		setXMLInt(xml, "assignments.preferedAssignmentTypeIndex", self.preferedAssignmentTypeIndex);
		saveXMLFile(xml);
		delete(xml);
	end;
	
	if self.currentAssignment ~= nil then
		self.currentAssignment.type.cleanUpFunction(self.currentAssignment);
	end;
	self.firstRunDone = nil;
end;

function FieldWorkAssignments:mouseEvent(posX, posY, isDown, isUp, button)

end;

function FieldWorkAssignments:keyEvent(unicode, sym, modifier, isDown)

end;

function FieldWorkAssignments:update(dt)
	if self.mapSupported then
		if InputBinding.isPressed(InputBinding.FIELDWORKMISSIONSKEY) then
			g_currentMission:addHelpButtonText(g_i18n:getText("openAssignmentMenu"), InputBinding.FIELDWORKMISSIONSSTARTMISSION);
			if InputBinding.hasEvent(InputBinding.FIELDWORKMISSIONSSTARTMISSION) then
				if self.assignmentSettingsHUD.gui.baseElement.visibility then
					self.assignmentSettingsHUD:close();
				else
					self.assignmentSettingsHUD:open();
				end;
			end;
		end;
		
		if not self.fileFound then
			self.fileFound = true;
			if not g_currentMission.connectedToDedicatedServer then
				self.welcomeGUI:open();
			end;
		end;
		
		if not self.firstUpdateRunDone then
			printContent(g_currentMission.missionInfo.mods, 4);
			self.assignmentSettingsHUD:setSelectedAssignmentTypeIndex(self.preferedAssignmentTypeIndex);
			self.firstUpdateRunDone = true;
			
			if g_server == nil then
				g_client:getServerConnection():sendEvent(JoinRequestAssignmentEvent:new());
			end;
		end;
		
		if g_server ~= nil then
			if table.getn(g_currentMission.users) ~= self.mpUserbackup then
				self.mpUserbackup = table.getn(g_currentMission.users);
				if self.currentAssignment ~= nil then
					for k, user in pairs(g_currentMission.users) do
						if not user.assignmentSynchronised and user ~= g_currentMission.users[g_currentMission.playerUserId] then 
							user.assignmentSynchronised = true;
							local assignment = self.currentAssignment;
							local stationName = "";
							if assignment.delivery ~= nil then
								stationName = assignment.delivery.stationName;
							end;
							user.connection:sendEvent(BroadcastAssignmentEvent2:new(assignment.fieldIndex, assignment.type.index, assignment.haToWorkOn, assignment.fruitType, stationName));
						end;
					end;
				end;
			end;
		end;
		
		if (self.cleanUpAssignmentsTimer > 0) then
			self.cleanUpAssignmentsTimer = self.cleanUpAssignmentsTimer - dt;
			if self.cleanUpAssignmentsTimer <= 0 then
				self.cleanUpAssignmentsTimer = -1;
				self:cleanUpAssignments();
			end;
		end;
	end;
	
	-- field definition test for mappers
	--[[if self.currentAssignment ~= nil then
		local field = self.currentAssignment.field;
		for a=1, table.getn(field.corners) do
			local corner = field.corners[a];
			drawDebugPoint(corner[1], getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, corner[1], 0, corner[2])+1, corner[2], 1, 0, 0, 1);
			drawDebugPoint(corner[3], getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, corner[3], 0, corner[4])+1, corner[4], 1, 0, 0, 1);
			drawDebugPoint(corner[5], getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, corner[5], 0, corner[6])+1, corner[6], 1, 0, 0, 1);
		end;
		
		for a=1, table.getn(field.corners) do
			local corner = field.corners[a];
			local ax, az = corner[3], corner[4];
			local bx, bz = corner[5], corner[6];
			local cx, cz = corner[1], corner[2];
			local abx, abz = bx-ax, bz-az;
			local acx, acz = cx-ax, cz-az;
			
			drawDebugLine(ax, getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, ax, 0, az)+1, az, 1, 0, 0, cx, getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, cx, 0, cz)+1, cz, 1, 0, 0);
			drawDebugPoint(bx, getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, bx, 0, bz)+1, bz, 1, 0, 0, 1);
		end;
	end;]]
end;

function FieldWorkAssignments:draw()

end;

function FieldWorkAssignments:handleAssignmentRequest(connection)
	local assignment, errorMessage = self:generateAssignment();
	
	if assignment ~= nil then -- assignment successfully generated
		if not g_currentMission.connectedToDedicatedServer then -- we are not a dedicated server
			g_currentMission.assignmentGUI:openAssignment(assignment); -- open assignmentGUI so the host can choose
		else -- we are a dedicated server - send assignment to all admins so they can choose
			local send = false;
			for a=1, table.getn(g_currentMission.users) do
				local user = g_currentMission.users[a];
				if user.isMasterUser then
					local stationName = "";
					if assignment.delivery ~= nil then
						stationName = assignment.delivery.stationName;
					end;
					user.connection:sendEvent(SuggestAssignmentEvent:new(assignment.fieldIndex, assignment.type.index, assignment.haToWorkOn, assignment.fruitType, stationName));
					send = true;
				end;
			end;		
			if not send then
				print("ERROR: Tried to send assignment to admins, no admin available");
			end;
		end;
		self.suggestedAssignment = assignment;
	else -- could not generate assignment
		if not g_currentMission.connectedToDedicatedServer then
			self.assignmentSettingsHUD:setErrorMessage(errorMessage);
		else -- send errorMessage to the admin who requested the assignment
			connection:sendEvent(SuggestAssignmentEvent:new(false, errorMessage));
		end;
	end;
end;

function FieldWorkAssignments:generateAssignment()
	if self.currentAssignment ~= nil then
		return nil, g_i18n:getText("otherAssignmentActive");
	end;

	math.randomseed(tonumber(getDate("%S")));

	-- select random field which the player does not own
	local unownedFields = {}
	for fieldIndex=1, table.getn(g_currentMission.fieldDefinitionBase.fieldDefs) do
		local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
		field.fieldIndex = fieldIndex;
		if not field.ownedByPlayer and field.useForAssignments then
			table.insert(unownedFields, field);
		end;
	end;
	local field;
	if table.getn(unownedFields) > 0 then
		field = unownedFields[math.random(1, table.getn(unownedFields))];
	else
		return nil, g_i18n:getText("noUnownedField");
	end;
	
	-- select assignment type and fruit type which is supported by the vehicles
	local assignmentType;
	local fruitType;
	if self.preferedAssignmentTypeIndex == 0 then -- random
		local possibleAssignmentTypes = {}
		for a=1, table.getn(AssignmentTypes) do
			local tmpType = AssignmentTypes[a];
			if tmpType.possibleFruitTypes ~= nil then
				for b=1, table.getn(tmpType.possibleFruitTypes) do
					local tmpFruit = FruitUtil.fruitTypes[tmpType.possibleFruitTypes[b]].index;
					if tmpType.checkVehicleFunction(tmpFruit) then
						table.insert(possibleAssignmentTypes, {assignmentType=tmpType, fruitType=tmpFruit});
					end;
				end;
			else
				if tmpType.checkVehicleFunction() then
					table.insert(possibleAssignmentTypes, {assignmentType=tmpType});
				end;
			end;
		end;
		if table.getn(possibleAssignmentTypes) == 0 then
			return nil, g_i18n:getText("NoAssignmentBuyVehicles");
		else
			local i = math.random(1, table.getn(possibleAssignmentTypes));
			assignmentType = possibleAssignmentTypes[i].assignmentType;
			if possibleAssignmentTypes[i].fruitType ~= nil then
				fruitType = possibleAssignmentTypes[i].fruitType;
			end;
		end;
	else
		assignmentType = AssignmentTypes[self.preferedAssignmentTypeIndex];
		local possibleFruits = {}
		if assignmentType.possibleFruitTypes ~= nil then
			for a=1, table.getn(assignmentType.possibleFruitTypes) do
				local tmpFruit = FruitUtil.fruitTypes[assignmentType.possibleFruitTypes[a]].index;
				if assignmentType.checkVehicleFunction(FruitUtil.fruitTypes[tmpFruit]) then
					table.insert(possibleFruits, tmpFruit);
				end;
			end;
			if table.getn(possibleFruits) == 0 then
				return nil, g_i18n:getText("NoAssignmentBuyVehicles");
			else
				fruitType = possibleFruits[math.random(1, table.getn(possibleFruits))];
			end;
		else
			if not assignmentType.checkVehicleFunction() then
				return nil, g_i18n:getText("NoAssignmentBuyVehicles");
			end;
		end;
	end;
	
	if assignmentType == nil then -- safety barrier, should never be called
		print("ERROR: ASSIGNMENT TYPE DOES NOT EXIST!");
		return nil, "FAIL!!";
	end;
	
	local _, fieldArea, fullArea = assignmentType.areaFunction(field.fieldIndex, fruitType);
	
	local triggers, stationName;
	if assignmentType.checkDeliveryFunction ~= nil then
		triggers, stationName = assignmentType.getDeliveryStation(fruitType);
		if triggers[1] == nil then
			return nil, g_i18n:getText("noTriggerFound");
		end;
	end;
	
	local assignment = {}
	assignment.fieldIndex = field.fieldIndex;
	assignment.field = field;
	assignment.type = assignmentType;
	assignment.totalArea = fieldArea;
	assignment.processedArea = 0; 
	assignment.fullArea = fullArea;
	assignment.fruitType = Utils.getNoNil(fruitType, FruitUtil.FRUITTYPE_UNKNOWN);
	assignment.haToWorkOn = field.fieldArea;
	assignment.price = math.floor(assignment.haToWorkOn * assignmentType.pricePerHa);
	assignment.progress = 0;
	
	if assignment.type.checkDeliveryFunction ~= nil then
		local fillType = FruitUtil.fruitTypeToFillType[assignment.fruitType];
		assignment.delivery = {}
		assignment.delivery.progress = 0;
		assignment.delivery.harvestedAmount = 0;
		assignment.delivery.deliveredAmount = 0;
		assignment.delivery.expectedFillType = fillType;
		assignment.delivery.stationName = stationName;
		assignment.delivery.surchargeFactor = 0.3;
		assignment.delivery.surcharge = math.floor(assignment.haToWorkOn * assignmentType.pricePerHa * assignment.delivery.surchargeFactor);
		assignment.delivery.triggerCheck = {} 
		assignment.delivery.triggers = triggers;
		for k, trigger in pairs(triggers) do
			assignment.delivery.triggerCheck[trigger] = true;
		end;
		
		if assignment.type.areaFunction == self.assignments.getFieldAreaToChop then
			assignment.delivery.expectedFillType = Fillable.fillTypeNameToInt["chaff"];
		end;
	end;

	assignment.briefing = self:getAssignmentBriefing(assignment);

	return assignment, "";
end;

function FieldWorkAssignments:getAssignmentBriefing(assignment)
	local text = g_i18n:getText("assignment") .. ": " .. assignment.type.description;
	
	if assignment.fruitType ~= nil and assignment.fruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
		local fruitType = g_i18n:getText(FruitUtil.fruitIndexToDesc[assignment.fruitType].name);
		text = string.gsub (text, "fruit", fruitType); 
	end;
	
	local fieldNumber = g_currentMission.fieldDefinitionBase.fieldDefs[assignment.fieldIndex].fieldNumber;
	if fieldNumber ~= nil then
		text = string.gsub (text, "fieldIndex", tostring(fieldNumber));
	end;
	
	if assignment.delivery ~= nil and assignment.delivery.stationName ~= nil then
		local stationName = assignment.delivery.stationName;
		local stationText;
		if g_i18n:hasText(stationName) then
			stationText = g_i18n:getText(stationName);
		elseif g_currentMission.fieldWorkAssignmentsMapg_18n ~= nil and g_currentMission.fieldWorkAssignmentsMapg_18n:hasText(stationName) then
			stationText = g_currentMission.fieldWorkAssignmentsMapg_18n:getText(stationName);
		else
			stationText = stationName;
		end;
		
		text = string.gsub (text, "station", stationText);
	end;
	return text;
end;

function FieldWorkAssignments:checkAssignment()
	local assignment = self.currentAssignment;
	local completed = false;
	
	local area, totalArea = assignment.type.areaFunction(assignment.fieldIndex, assignment.fruitType);
	assignment.processedArea = area;
	completed = (assignment.fullArea-assignment.processedArea) < (assignment.fullArea * self.assignmentWorkTolerance);
	assignment.progress = (assignment.processedArea/assignment.fullArea)*100;
		
	local deliveryCompleted = true;
	if assignment.type.checkDeliveryFunction ~= nil then
		deliveryCompleted = assignment.type.checkDeliveryFunction(assignment);
	end;
		
	self.assignmentHUD:updateHUDValues();
	
	if completed and deliveryCompleted then
		assignment.progress = 100;
		if assignment.delivery ~= nil then
			assignment.delivery.progress = 100;
		end;
	
		self:finishAssignment();
	else
		g_server:broadcastEvent(UpdateAssignmentProgressEvent:new(assignment), nil, nil, nil);
		return true;
	end;
end;

function FieldWorkAssignments:acceptAssignment(assignment)
	self.currentAssignment = assignment;

	if assignment.type.startAssignmentCallback ~= nil then
		assignment.type.startAssignmentCallback(assignment);
	end;
	
	--if g_client ~= nil then
		self.assignmentHUD:open(assignment);
	--end;
	
	if g_server ~= nil then
		self.checkAssignmentTimer = addTimer(1000, "checkAssignment", self);
		local stationName = "";
		if assignment.delivery ~= nil then
			stationName = assignment.delivery.stationName;
		end;
		g_server:broadcastEvent(BroadcastAssignmentEvent2:new(assignment.fieldIndex, assignment.type.index, assignment.haToWorkOn, assignment.fruitType, stationName));
		for a=1, table.getn(g_currentMission.users) do
			g_currentMission.users[a].assignmentSynchronised = true;
		end;		
	end;
	
	g_currentMission.fieldDefinitionBase:setFieldOwnedByPlayer(assignment.field, true);
end;

function FieldWorkAssignments:finishAssignment()
	if g_server ~= nil then
		local assignmentPrice = self.currentAssignment.price*self.currentAssignment.progress/100;
		local deliveryPrice = 0;
		if self.currentAssignment.delivery ~= nil then
			deliveryPrice = self.currentAssignment.delivery.surcharge*self.currentAssignment.delivery.progress/100*self.currentAssignment.progress/100;
		end;
		g_currentMission:addSharedMoney(assignmentPrice+deliveryPrice, "missionIncome"); 
		g_server:broadcastEvent(FinishAssignmentEvent:new(), nil, nil, nil);
	end;
	self.assignmentHUD:close();
	self.assignmentCompletedGui:open(self.currentAssignment);
	table.insert(self.assignmentsToCleanUp, self.currentAssignment);
	g_currentMission.fieldDefinitionBase:setFieldOwnedByPlayer(self.currentAssignment.field, false);
	
	self.cleanUpAssignmentsTimer = 5000;
	self.currentAssignment = nil;
end;

function FieldWorkAssignments:cancelAssignment()
	if g_server ~= nil then
		removeTimer(self.checkAssignmentTimer);
		self:finishAssignment();
	end;
end;

function FieldWorkAssignments:setPreferedAssignmentTypeIndex(index)
	self.preferedAssignmentTypeIndex = index;
end;

function FieldWorkAssignments:cleanUpAssignments()
	for a=1, table.getn(self.assignmentsToCleanUp) do
		local assignment = self.assignmentsToCleanUp[a];
		if assignment.type.cleanUpFunction ~= nil then
			assignment.type.cleanUpFunction(assignment);
		end;
	end;
	self.assignmentsToCleanUp = {}
	return false;
end;


function FieldWorkAssignments:getIsVehicleOnField(fieldIndex, px, pz)
	local field = g_currentMission.fieldDefinitionBase.fieldDefs[fieldIndex];
	for a=1, table.getn(field.corners) do
		local corner = field.corners[a];
		local ax, az = corner[3], corner[4];
		local bx, bz = corner[5], corner[6];
		local cx, cz = corner[1], corner[2];
		local abx, abz = bx-ax, bz-az;
		local acx, acz = cx-ax, cz-az;
		
		local AC_b = FieldWorkAssignments.getPointToLine({ax, az}, {acx, acz}, {bx, bz});
		local AB_c = FieldWorkAssignments.getPointToLine({ax, az}, {abx, abz}, {cx, cz});
		
		local AC_p = FieldWorkAssignments.getPointToLine({ax, az}, {acx, acz}, {px, pz});
		local AB_p = FieldWorkAssignments.getPointToLine({ax, az}, {abx, abz}, {px, pz});
		if AC_p <= AC_b and AB_p <= AB_c then
			return true;
		end;
	end;
	return false;
end;

function FieldWorkAssignments.getPointToLine(a, u, p) -- a=ortsvektor; u=richtungsvektor, p=punkt
	-- g: x = a+r*u
	local zaehler = ((p[1]-a[1])*u[2]) - (u[1]*(p[2]-a[2]));
	local nenner = Utils.vector2Length(u[1], u[2]);
	return math.abs(zaehler) / math.abs(nenner);
end;

addModEventListener(FieldWorkAssignments);

-- save fix
local oldFSCareerMissionInfoSaveToXML = FSCareerMissionInfo.saveToXML;
FSCareerMissionInfo.saveToXML = function(...)
	if g_currentMission.fieldWorkMissions.currentAssignment ~= nil then
		g_currentMission.fieldDefinitionBase:setFieldOwnedByPlayer(g_currentMission.fieldWorkMissions.currentAssignment.field, false);
	end;
	oldFSCareerMissionInfoSaveToXML(...);
	if g_currentMission.fieldWorkMissions.currentAssignment ~= nil then
		g_currentMission.fieldDefinitionBase:setFieldOwnedByPlayer(g_currentMission.fieldWorkMissions.currentAssignment.field, true);
	end;
end;